'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _Chips = require('../Chips');

var _Chips2 = _interopRequireDefault(_Chips);

var _module = require('module');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('chips', function () {
	var saveCallback = jest.fn();
	var props = {
		chips: [],
		title: 'component title',
		save: saveCallback,
		placeholder: 'component input placeholder',
		pattern: new RegExp(/^\w+([\.-]?\w+)*@\w+([\.-]?\w+)*(\.\w{2,3})+$/),
		required: true
	};

	var mockChips = [{ email: 'react@gmail.com', valid: true, key: '1' }, { email: 'javascript@gmail.com', valid: true, key: '2' }, { email: 'scss@gmail.com', valid: true, key: '3' }];

	test('should render', function () {
		expect((0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props))).toHaveLength(1);
	});

	describe('focus input', function () {
		test('should set focus on input when clicking inside chips container', function () {
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props));
			wrapper.instance().focusInput = jest.fn();
			wrapper.find('.chips').at(0).simulate('click');
			expect(wrapper.instance().focusInput).toHaveBeenCalled();
		});
	});

	describe('set chips', function () {
		test('should set chips when component renders', function () {
			var spy = jest.spyOn(_Chips2.default.prototype, 'setChips');
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, _extends({}, props, { chips: mockChips })));
			expect(wrapper.instance().setChips).toHaveBeenCalledWith(mockChips, false);
			spy.mockRestore();
		});

		test('should set state and not call save callback', function () {
			var saveCallback = jest.fn();
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, _extends({}, props, { save: saveCallback })));
			expect(wrapper.state('chips')).toHaveLength(0);
			wrapper.instance().setChips(mockChips);
			expect(wrapper.state('chips')).toHaveLength(3);
			expect(saveCallback).not.toHaveBeenCalled();
		});

		test('should set state and call save callback', function () {
			var saveCallback = jest.fn();
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, _extends({}, props, { save: saveCallback })));
			expect(wrapper.state('chips')).toHaveLength(0);
			wrapper.instance().setChips(mockChips, true);
			expect(wrapper.state('chips')).toHaveLength(3);
			expect(saveCallback).toHaveBeenCalledTimes(1);
		});

		test('get valid chips', function () {
			var mock = mockChips.concat([{ email: 'fsdfsdfds', valid: false, key: 10 }]);
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props));
			var result = wrapper.instance().getValidChips(mock);
			expect(result).toHaveLength(3);
		});
	});

	describe('on key down', function () {
		var updateChipsSpy = void 0,
		    deleteChipsSpy = void 0,
		    wrapper = void 0;

		beforeAll(function () {
			updateChipsSpy = jest.spyOn(_Chips2.default.prototype, 'updateChips');
			deleteChipsSpy = jest.spyOn(_Chips2.default.prototype, 'deleteChip');
		});

		afterAll(function () {
			updateChipsSpy.mockRestore();
			deleteChipsSpy.mockRestore();
		});

		beforeEach(function () {
			wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props));
		});
		afterEach(function () {
			updateChipsSpy.mockClear();
			deleteChipsSpy.mockClear();
		});

		test('clear required validation', function () {
			wrapper.setState({
				requiredValidation: true
			});
			expect(wrapper.state('requiredValidation')).toBe(true);
			wrapper.instance().clearRequiredValidation();
			expect(wrapper.state('requiredValidation')).toBe(false);
		});

		test('should clear required validation', function () {
			var event = { which: 64, target: { value: '' } };
			wrapper.instance().clearRequiredValidation = jest.fn();
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().clearRequiredValidation).toHaveBeenCalledTimes(1);
		});

		test('should call update chips with enter key', function () {
			var event = { which: 13, target: { value: '' }, preventDefault: jest.fn() };
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().updateChips).toHaveBeenCalledWith(event);
		});

		test('should call update chips with tab key', function () {
			var event = { which: 9, target: { value: 'tmp' }, preventDefault: jest.fn() };
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().updateChips).toHaveBeenCalledWith(event);
		});

		test('should not call update chips if tab key was pressed with no value', function () {
			var event = { which: 9, target: { value: '' }, preventDefault: jest.fn() };
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().updateChips).not.toHaveBeenCalled();
		});

		test('should call delete chip if backspace was clicked and there is chips and value is null', function () {
			var event = { which: 8, target: { value: '' } };
			wrapper.instance().setChips(mockChips);
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().deleteChip).toHaveBeenCalled();
		});

		test('should not call delete chip if backspace was clicked and value is not null', function () {
			var event = { which: 8, target: { value: 'value' } };
			wrapper.instance().setChips(mockChips);
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().deleteChip).not.toHaveBeenCalled();
		});

		test('should not call delete chip if backspace was clicked and there is no chips', function () {
			var event = { which: 8, target: { value: '' } };
			wrapper.instance().onKeyDown(event);
			expect(wrapper.instance().deleteChip).not.toHaveBeenCalled();
		});
	});

	describe('delete chip', function () {
		var wrapper = void 0;
		beforeEach(function () {
			wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props));
		});

		test('should return undefined if there no removedChip', function () {
			var result = wrapper.instance().deleteChip(undefined);
			expect(result).toBe(undefined);
		});

		test('should set requiredValidation if prop required and removed chip is valid and there is only one valid email', function () {
			wrapper.setState({ requiredValidation: false });
			wrapper.instance().setChips(mockChips.slice(2));
			wrapper.instance().deleteChip(mockChips[2]);
			expect(wrapper.state('requiredValidation')).toBe(true);
		});

		test('should not  set requiredValidation if prop required is false', function () {
			wrapper.setState({ requiredValidation: false });
			wrapper.setProps({ required: false });
			wrapper.instance().setChips(mockChips.slice(2));
			wrapper.instance().deleteChip(mockChips[2]);
			expect(wrapper.state('requiredValidation')).toBe(false);
		});

		test('should not set requiredValidation if email is not valid', function () {
			wrapper.setState({ requiredValidation: false });
			var chips = mockChips.map(function (chip, index) {
				return _extends({}, chip, { valid: Boolean(index % 2) });
			});
			wrapper.setState({ requiredValidation: false });
			wrapper.instance().setChips(chips);
			wrapper.instance().deleteChip(chips[0]);
			expect(wrapper.state('requiredValidation')).toBe(false);
		});

		test('should not set requiredValidation if there is more then one email', function () {
			wrapper.setState({ requiredValidation: false });
			var chips = mockChips.map(function (chip, index) {
				return _extends({}, chip, { valid: Boolean(!(index % 2)) });
			});

			wrapper.instance().setChips(chips);
			wrapper.instance().deleteChip(chips[0]);
			expect(wrapper.state('requiredValidation')).toBe(false);
		});

		test('should call setChips with the new list and save flag true', function () {
			wrapper.instance().setChips(mockChips);
			wrapper.instance().setChips = jest.fn();
			wrapper.instance().deleteChip(mockChips[0]);

			expect(wrapper.instance().setChips).toHaveBeenCalledWith(mockChips.slice(1), true);
		});

		test('should call setChips with the new list and save flag false', function () {
			var newMockChips = [].concat(mockChips);
			newMockChips[0] = _extends({}, newMockChips[0], { valid: false });
			wrapper.instance().setChips(newMockChips);
			wrapper.instance().setChips = jest.fn();
			wrapper.instance().deleteChip(newMockChips[0]);

			expect(wrapper.instance().setChips).toHaveBeenCalledWith(newMockChips.slice(1), false);
		});
	});

	describe('is only one valid chip', function () {
		test('should return true when only one chip is valid', function () {
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props));
			var chips = mockChips.map(function (chip, index) {
				return _extends({}, chip, { valid: Boolean(index % 2) });
			});
			wrapper.instance().setChips(chips);
			var result = wrapper.instance().isOnlyOneValidChip();
			expect(result).toBe(true);
		});

		test('should return false when more then  one chip is valid', function () {
			var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, props));
			var chips = mockChips.map(function (chip, index) {
				return _extends({}, chip, { valid: Boolean(!(index % 2)) });
			});
			wrapper.instance().setChips(chips);
			var result = wrapper.instance().isOnlyOneValidChip();
			expect(result).toBe(false);
		});
	});

	describe('updateChips', function () {
		var instance = void 0;
		beforeEach(function () {
			instance = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, _extends({}, props, { chips: mockChips }))).instance();
			instance.setChips = jest.fn();
		});

		test('should return undefined when value is empty', function () {
			var event = { target: { value: '' } };
			var result = instance.updateChips(event);
			expect(result).toBe(undefined);
		});

		test('should return undefined if chip already exists', function () {
			var event = { target: { value: 'react@gmail.com' } };
			var result = instance.updateChips(event);
			expect(result).toBe(undefined);
		});

		test('should set valid as true when props.pattern is undefined', function () {
			var event = { target: { value: 'tmpexamplecom' } };
			var newProps = _extends({}, props);
			delete newProps.pattern;
			instance = (0, _enzyme.shallow)(_react2.default.createElement(_Chips2.default, _extends({}, newProps, { chips: mockChips }))).instance();
			instance.setChips = jest.fn();

			instance.updateChips(event);

			var _instance$setChips$mo = instance.setChips.mock.calls.slice(-1),
			    _instance$setChips$mo2 = _slicedToArray(_instance$setChips$mo, 1),
			    _instance$setChips$mo3 = _slicedToArray(_instance$setChips$mo2[0], 1),
			    lastCall = _instance$setChips$mo3[0];

			var _lastCall$slice = lastCall.slice(-1),
			    _lastCall$slice2 = _slicedToArray(_lastCall$slice, 1),
			    chip = _lastCall$slice2[0];

			expect(chip.email).toEqual('tmpexamplecom');
			expect(chip.valid).toBe(true);
			expect(true).toBe(true);
		});

		test('should add new valid chip ', function () {
			var event = { target: { value: 'tmp@example.com' } };
			instance.updateChips(event);

			var _instance$setChips$mo4 = instance.setChips.mock.calls.slice(-1),
			    _instance$setChips$mo5 = _slicedToArray(_instance$setChips$mo4, 1),
			    _instance$setChips$mo6 = _slicedToArray(_instance$setChips$mo5[0], 1),
			    lastCall = _instance$setChips$mo6[0];

			var _lastCall$slice3 = lastCall.slice(-1),
			    _lastCall$slice4 = _slicedToArray(_lastCall$slice3, 1),
			    chip = _lastCall$slice4[0];

			expect(chip.email).toEqual('tmp@example.com');
			expect(chip.valid).toBe(true);
		});

		test('should add new invalid chip', function () {
			var event = { target: { value: 'tmpexamplecom' } };
			instance.updateChips(event);

			var _instance$setChips$mo7 = instance.setChips.mock.calls.slice(-1),
			    _instance$setChips$mo8 = _slicedToArray(_instance$setChips$mo7, 1),
			    _instance$setChips$mo9 = _slicedToArray(_instance$setChips$mo8[0], 1),
			    lastCall = _instance$setChips$mo9[0];

			var _lastCall$slice5 = lastCall.slice(-1),
			    _lastCall$slice6 = _slicedToArray(_lastCall$slice5, 1),
			    chip = _lastCall$slice6[0];

			expect(chip.email).toEqual('tmpexamplecom');
			expect(chip.valid).toBe(false);
		});
	});
});