'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _enzyme = require('enzyme');

var _ChipsList = require('../ChipsList');

var _ChipsList2 = _interopRequireDefault(_ChipsList);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var chipsList = [{
	email: 'tmp@example.com',
	valid: true,
	key: '1'
}, {
	email: 'tmp-2@example.com',
	valid: true,
	key: '2'
}, {
	email: 'bad-email-value',
	valid: false,
	key: '3'
}];
describe('ChipsList', function () {
	test('should render', function () {
		expect((0, _enzyme.shallow)(_react2.default.createElement(_ChipsList2.default, { chips: chipsList }))).toHaveLength(1);
	});

	test('should set valid / invalid email', function () {
		var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_ChipsList2.default, { chips: chipsList }));
		expect(wrapper.find('.chip').at(0).find('.not-valid')).toHaveLength(0);
		expect(wrapper.find('.chip').at(1).find('.not-valid')).toHaveLength(0);
		expect(wrapper.find('.chip').at(2).find('.not-valid')).toHaveLength(1);
	});

	test('should respond to click with chip', function () {
		var callback = jest.fn();
		var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_ChipsList2.default, { chips: chipsList, onChipClick: callback }));
		wrapper.find('.chip').at(0).find('button').first().props().onClick();
		expect(callback).toHaveBeenCalled();
		expect(callback).toHaveBeenCalledWith(chipsList[0]);
	});

	test('component should not render when passing same props', function () {
		var spy = jest.spyOn(_ChipsList2.default.prototype, 'render');
		var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_ChipsList2.default, { chips: chipsList }));
		expect(wrapper.find('.chip')).toHaveLength(3);
		wrapper.setProps({ chips: chipsList });
		expect(_ChipsList2.default.prototype.render).toHaveBeenCalled();
		spy.mockRestore();
	});

	test('component should render when passing different props', function () {
		var wrapper = (0, _enzyme.shallow)(_react2.default.createElement(_ChipsList2.default, { chips: chipsList }));
		var spy = jest.spyOn(_ChipsList2.default.prototype, 'render');
		expect(wrapper.find('.chip')).toHaveLength(3);
		wrapper.setProps({ chips: chipsList.slice(0, 1) });
		expect(_ChipsList2.default.prototype.render).toHaveBeenCalled();
		spy.mockRestore();
	});
});